
#include <math.h>
#include <GL/glut.h>
#include <stdlib.h>
#include <time.h>
#include <iostream>

using namespace std;

////////////////////////////////////////
//  Light methods
////////////////////////////////////////

GLfloat lightAmbient[]= { 0.2f, 0.2f, 0.2f, 1.0f };
GLfloat lightDiffuse[]= { 1.0f, 1.0f, 1.0f, 1.0f };
float lightAngle = 0.0f;

void setLight(){

                GLfloat lightPosition[]= { 4.0*cos(lightAngle), 3.0f, 4.0*sin(lightAngle), 1.0f };
                glLightfv(GL_LIGHT0, GL_AMBIENT, lightAmbient);
                glLightfv(GL_LIGHT0, GL_DIFFUSE, lightDiffuse);
                glLightfv(GL_LIGHT0, GL_POSITION,lightPosition);
                glEnable(GL_LIGHT0);
}




/////////////////////////////////////////
//  Camera methods
////////////////////////////////////////

float cameraAngle=M_PI*0.5f, distToOrigin = 10.0, cameraHeight = 3.0f;

void setCamera() {

                float x = distToOrigin*cos(cameraAngle);
                float y = cameraHeight;
                float z = distToOrigin*sin(cameraAngle);

                glLoadIdentity();
                gluLookAt(x, y, z,
                                                        -x, y, -z,
                                                        0.0f,1.0f,0.0f);
                setLight();
}


////////////////////////////////////////
// Initialization methods
////////////////////////////////////////

void initScene() {
                glClearColor(0.0f, 0.0f, 0.0f, 0.0f);                          // This Will Clear The Background Color To Black
                glClearDepth(1.0);                                             // Enables Clearing Of The Depth Buffer
                glDepthFunc(GL_LESS);                                          // The Type Of Depth Test To Do
                glEnable(GL_DEPTH_TEST);                                       // Enables Depth Testing
                glShadeModel(GL_SMOOTH);                                       // Enables Smooth Color Shading
                glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);      // Controls the rendering style (fill, line or point)
                glLineWidth(2.0f);
                glPointSize(4.0f);

                // init lighting
                setLight();
                // Uncomment to use lighting
                // glEnable(GL_LIGHTING);
}


void changeSize(int w, int h)   {

                // Prevent a divide by zero, when window is too short
                // (you cant make a window of zero width).
                if(h == 0)
                                h = 1;

                float ratio = 1.0f * w / h;
                // Reset the coordinate system before modifying
                glMatrixMode(GL_PROJECTION);
                glLoadIdentity();

                // Set the viewport to be the entire window
                glViewport(0, 0, w, h);

                // Set the clipping volume
                gluPerspective(45,ratio,1,1000);
                glMatrixMode(GL_MODELVIEW);
                setCamera();
}










///////////////////////////////////////////////
//  Draw methods
//////////////////////////////////////////////

void drawGround(){

                float backColor[] = {1.0, 0.0, 0.0};
                float frontColor[] = {0.0, 0.0, 1.0};

                glBegin(GL_QUADS);

        glColor3fv(frontColor);
                glVertex3f(-10.0f, 0.0f, -10.0f);
        glColor3fv(backColor);
                glVertex3f(-10.0f, 0.0f,  10.0f);
        glColor3fv(backColor);
                glVertex3f( 10.0f, 0.0f,  10.0f);
        glColor3fv(frontColor);
                glVertex3f( 10.0f, 0.0f, -10.0f);

                glEnd();
}


void drawCube(){

   glBegin(GL_QUADS);

   glNormal3f(0,0,1);
         glColor3f(0.0, 0.0, 1.0);
         glVertex3f(-1.0,-1.0,1.0);
         glColor3f(0.0, 0.0, 1.0);
         glVertex3f(1.0,-1.0,1.0);
         glColor3f(1.0, 0.0, 0.0);
         glVertex3f(1.0,1.0,1.0);
         glColor3f(1.0, 0.0, 0.0);
         glVertex3f(-1.0,1.0,1.0);

         glNormal3f(0,0,-1);
         glColor3f(1.0, 0.0, 0.0);
         glVertex3f(-1.0,1.0,-1.0);
         glColor3f(1.0, 0.0, 0.0);
         glVertex3f(1.0,1.0,-1.0);
         glColor3f(0.0, 0.0, 1.0);
         glVertex3f(1.0,-1.0,-1.0);
         glColor3f(0.0, 0.0, 1.0);
         glVertex3f(-1.0,-1.0,-1.0);

   glNormal3f(-1,0,0);
         glColor3f(0.0, 0.0, 1.0);
         glVertex3f(-1.0,-1.0,1.0);
         glColor3f(1.0, 0.0, 0.0);
         glVertex3f(-1.0,1.0,1.0);
         glColor3f(1.0, 0.0, 0.0);
         glVertex3f(-1.0,1.0,-1.0);
         glColor3f(0.0, 0.0, 1.0);
         glVertex3f(-1.0,-1.0,-1.0);

         glNormal3f(1,0,0);
         glColor3f(0.0, 0.0, 1.0);
         glVertex3f(1.0,-1.0,-1.0);
         glColor3f(1.0, 0.0, 0.0);
         glVertex3f(1.0,1.0,-1.0);
         glColor3f(1.0, 0.0, 0.0);
         glVertex3f(1.0,1.0,1.0);
         glColor3f(0.0, 0.0, 1.0);
         glVertex3f(1.0,-1.0,1.0);

         glNormal3f(0,-1,0);
         glColor3f(0.0, 0.0, 1.0);
         glVertex3f(-1.0,-1.0,-1.0);
         glColor3f(0.0, 0.0, 1.0);
         glVertex3f(1.0,-1.0,-1.0);
         glColor3f(0.0, 0.0, 1.0);
         glVertex3f(1.0,-1.0,1.0);
         glColor3f(0.0, 0.0, 1.0);
         glVertex3f(-1.0,-1.0,1.0);

         glNormal3f(0,1,0);
         glColor3f(1.0, 0.0, 0.0);
         glVertex3f(-1.0,1.0,1.0);
         glColor3f(1.0, 0.0, 0.0);
         glVertex3f(1.0,1.0,1.0);
         glColor3f(1.0, 0.0, 0.0);
         glVertex3f(1.0,1.0,-1.0);
         glColor3f(1.0, 0.0, 0.0);
         glVertex3f(-1.0,1.0,-1.0);

   glEnd();




}



void drawCone(){
                int nbFaces = 50;
                float stepAngle = 2*M_PI/nbFaces;

                // draw base
                glBegin(GL_POLYGON);
                glColor3f(1.0, 0.0, 0.0);
                glNormal3f(0.0, -1.0, 0.0);
                for (int i=0 ; i<nbFaces ; i++){
                                float angle = i*stepAngle;
                                glVertex3f(cos(angle), 0.0, sin(angle));
                }
                glEnd();


                // draw apex
                float norm = 0.5*sqrt(2.0);
                glBegin(GL_TRIANGLE_FAN);
                glColor3f(1.0, 0.0, 0.0);
                glNormal3f(0.0, 1.0, 0.0);
                glVertex3f(0.0, 1.0, 0.0);
                for (int i=0 ; i<=nbFaces ; i++){
                                float angle = i*stepAngle;
                                glColor3f(0.0, 0.0, 1.0);
                                glNormal3f(norm*cos(angle), norm, norm*sin(angle));
                                glVertex3f(cos(angle), 0.0, sin(angle));
                }
                glEnd();

}

void drawCylinder(){
                int nbStrips = 20;

                int nbFacesPerStrip = 50;
                float stepHAngle = 2*M_PI/nbFacesPerStrip;

                // draw top
                glBegin(GL_POLYGON);
                glColor3f(1.0, 0.0, 0.0);
                glNormal3f(0.0, 1.0, 0.0);
                for (int i=0 ; i<nbFacesPerStrip ; i++){
                                float angle = i*stepHAngle;
                                glVertex3f(cos(angle), 1.0, sin(angle));
                }
                glEnd();

                // draw base
                glBegin(GL_POLYGON);
                glColor3f(0.0, 0.0, 1.0);
                glNormal3f(0.0, -1.0, 0.0);
                for (int i=0 ; i<nbFacesPerStrip ; i++){
                                float angle = i*stepHAngle;
                                glVertex3f(cos(angle), -1.0, sin(angle));
                }
                glEnd();

                // draw cylinder
                for(int i=0 ; i<nbStrips ; i++) {
                                float y1 = 2.0*i/nbStrips - 1.0;
                                float y2 = 2.0*(i+1.0)/nbStrips - 1.0;

                                // draw one strip
                                float intensity1 = (float)i/nbStrips;
                                float intensity2 = (float)(i+1)/nbStrips;
                                glBegin(GL_QUAD_STRIP);
                                for (int j=0 ; j<=nbFacesPerStrip ; j++){
                                                float hAngle = j*stepHAngle;
                                                glNormal3f(cos(hAngle), 0.0, sin(hAngle));

                                                glColor3f(intensity1, 0.0, 1.0-intensity1);
                                                glVertex3f(cos(hAngle), y1, sin(hAngle));
                                                glColor3f(intensity2, 0.0, 1.0-intensity2);
                                                glVertex3f(cos(hAngle), y2, sin(hAngle));
                                }
                                glEnd();
                }

}

void drawSphere(){
                int nbStrips = 20;
                float stepVAngle = M_PI/nbStrips;

                int nbFacesPerStrip = 50;
                float stepHAngle = 2*M_PI/nbFacesPerStrip;

                for(int i=0 ; i<nbStrips ; i++) {
                                float vAngle1 = i*stepVAngle - 0.5*M_PI;
                                float vAngle2 = (i+1)*stepVAngle - 0.5*M_PI;

                                // draw one strip
                                float intensity1 = (float)i/nbStrips;
                                float intensity2 = (float)(i+1)/nbStrips;
                                glBegin(GL_QUAD_STRIP);
                                for (int j=0 ; j<=nbFacesPerStrip ; j++){
                                                float hAngle = j*stepHAngle;
                                                float pos1[] = {cos(hAngle)*cos(vAngle1), sin(vAngle1), sin(hAngle)*cos(vAngle1)};
                                                float pos2[] = {cos(hAngle)*cos(vAngle2), sin(vAngle2), sin(hAngle)*cos(vAngle2)};

                                                glColor3f(intensity1, 0.0, 1.0-intensity1);
                                                glNormal3fv(pos1);
                                                glVertex3fv(pos1);
                                                glColor3f(intensity2, 0.0, 1.0-intensity2);
                                                glNormal3fv(pos2);
                                                glVertex3fv(pos2);
                                }
                                glEnd();
                }

}


void drawRobot() {

                glPushMatrix();
                // TODO
                glPopMatrix();
}


void drawPrimitives(){
                glPushMatrix();

                glTranslatef(-4.0f ,1.0f, 0.0f);
                drawCube();
                glTranslatef(2.25f ,0.0f, 0.0f);
                drawCone();
                glTranslatef(2.25f ,0.0f, 0.0f);
                drawSphere();
                glTranslatef(2.25f ,0.0f, 0.0f);
                drawCylinder();

                glPopMatrix();
}


void renderScene(void) {
                // clear buffers
                glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

                // draw objects
                drawGround();
                drawPrimitives();


                // display rendering
                glutSwapBuffers();
}









///////////////////////////////////////
//  Interaction methods
///////////////////////////////////////


void specialKey(int key, int x_, int y_) {

                switch (key) {
                case GLUT_KEY_LEFT :
                                cameraAngle += 0.1f;
                                setCamera();  break;
                case GLUT_KEY_RIGHT :
                                cameraAngle -=0.1f;
                                setCamera();  break;
                case GLUT_KEY_UP :
                                distToOrigin -= 0.2f;
                                if (distToOrigin<1.5f) distToOrigin = 1.5f;
                                setCamera();  break;
                case GLUT_KEY_DOWN :
                                distToOrigin += 0.2f;
                                setCamera();  break;
                case GLUT_KEY_PAGE_UP :
                                cameraHeight += 0.1f;
                                setCamera();  break;
                case GLUT_KEY_PAGE_DOWN :
                                cameraHeight -= 0.1f;
                                setCamera();  break;
                case GLUT_KEY_F1 :
                                lightAngle += 0.1f;
                                setLight(); break;
                case GLUT_KEY_F2 :
                                lightAngle -= 0.1f;
                                setLight(); break;
                }
}




void commonKey(unsigned char key, int x_, int y_) {

                switch (key) {
                default:
                                cout << key << endl; break;
                }
}






////////////////////////////////////
//  Main method
////////////////////////////////////

int main(int argc, char **argv) {

                // create window
                glutInit(&argc, argv);
                glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
                glutInitWindowPosition(100,100);
                glutInitWindowSize(800, 600);
                glutCreateWindow("Humanoid");

                // initialize OpenGL states
                initScene();

                // Bind Key events
                glutSpecialFunc(specialKey);
                glutKeyboardFunc(commonKey);

                // Bind rendering methods
                glutDisplayFunc(renderScene);
                glutIdleFunc(renderScene);

                glutReshapeFunc(changeSize);

                // run program
                glutMainLoop();

                return(0);
}
