#include "dynamics.h"
#include <iostream>
#include <math.h>

using namespace std;

/////////////////////////
// accessors
/////////////////////////

int Dynamics::addBall(const Vec& ballPos, const Vec& velocity, float mass, float radius){
	positions.push_back(ballPos);
	velocities.push_back(velocity);
	masses.push_back(mass);
	if (mass>0) {
		invMasses.push_back( 1/mass);
	} else {
		invMasses.push_back(0.0);
	}
	radiuses.push_back(radius);

	return positions.size()-1;
}

int Dynamics::addSpring(int ball1, int ball2, double stiffness, double initLength, double dampeningFactor){
	springs.push_back(Spring(ball1, ball2, stiffness, initLength, dampeningFactor));
	return springs.size();
}

void Dynamics::setGravity(const Vec& grav){
	gravity = grav;
}

void Dynamics::setBallPos( int boule, const Vec& pos){
	if (boule<0 || boule>=positions.size()){
		cerr << "Bad ball index !!" << endl;
	}
	positions[boule] = pos;
}



/////////////////////////
// methods
/////////////////////////

void Dynamics::animate(double dt){

		////////////////////
		// calcul des forces
		
		int nbBalls = positions.size();
		vector<Vec> forces(nbBalls);
		for( int i=0; i<nbBalls; ++i ) {
				// poids
				forces[i] = forces[i] + masses[i] * gravity;
				// viscosite
				forces[i] = forces[i] - velocities[i] * viscosity;
		}
		
		for( int i=0; i<springs.size(); ++i ) {
				// ressorts amortis
				springs[i].addForce(forces, positions, velocities);
		}
		
		///////////////////////////
		// calcul des accelerations
		
		for( int i=0; i<nbBalls; ++i ) {
				// les acclrations sont stockes dans forces
				forces[i] = invMasses[i] * forces[i];
		}
	
		///////////////////////
		// schma d'intgration
		for( int i=0; i<nbBalls; ++i ) {
				positions[i] = positions[i] + dt * velocities[i] + dt*dt*0.5 * forces[i];
				velocities[i] = velocities[i] + dt * forces[i];
		}

		////////////////////////////
		// traitement des collisions
		
		for( int i=0; i<nbBalls; ++i ){
				// avec le sol
				collisionBallPlane(positions[i], velocities[i], radiuses[i], invMasses[i],
													 groundPosition, groundVelocity, groundNormal, 0.0f, 0.5f);
				
				// entre les boules
				for (int j=0 ; j<nbBalls ; j++){					
						if (i!=j){
								collisionBallBall(positions[i], velocities[i], radiuses[i], invMasses[i],
																	positions[j], velocities[j], radiuses[j], invMasses[j],
																	1.0f);
						}
				}
		}
		
}



void Dynamics::collisionBallPlane(Vec& x1, Vec& v1, double r1, double invm1,
																	Vec& x2, Vec& v2, Vec& normal, double invm2, double rebound ){
		// ne pas traiter les objets fixes
		if( invm1==0 && invm2==0 ) return;
	
		// calcul de la distance au plan
		Vec gap = x1 - x2;
		double penetration = (gap * normal) - r1;
		if( penetration > 0 ) return;
		
		// vitesse de penetration
		double vpen = (v1-v2)*normal;
		
		// rpartition de la correction (le plus lourd bouge moins)
		double corr1 = invm1 / (invm1+invm2);
		double corr2 = invm2 / (invm1+invm2);
		
		// prise en compte du rebond
		corr1 *= (1+rebound);
		corr2 *= (1+rebound);
		
		// mise  jour des positions et vitesses
		x1 = x1 - (corr1 * penetration) * normal;
		v1 = v1 - (corr1 * vpen) * normal;
		x2 = x2 + (corr2 * penetration) * normal;
		v2 = v2 + (corr2 * vpen) * normal;		
}


void Dynamics::collisionBallBall(Vec& x1, Vec& v1, double r1, double invm1,
																 Vec& x2, Vec& v2, double r2, double invm2,
																 double rebound ){
		// ne pas traiter les objets fixes
		if( invm1==0 && invm2==0 ) return;
	
		// calcul de la distance entre les boules
		Vec gap = x1 - x2;
		float distance = sqrt(gap*gap);
		double penetration = distance - (r1+r2);
		if( penetration > 0 ) return;
		
		// vitesse de penetration
		Vec normal = gap * (1.0/distance);
		double vpen = (v1-v2)*normal;
		
		// rpartition de la correction (le plus lourd bouge moins)
		double corr1 = invm1 / (invm1+invm2);
		double corr2 = invm2 / (invm1+invm2);
		
		// prise en compte du rebond
		corr1 *= (1+rebound);
		corr2 *= (1+rebound);
		
		// mise  jour des positions et vitesses
		x1 = x1 - (corr1 * penetration) * normal;
		v1 = v1 - (corr1 * vpen) * normal;
		x2 = x2 + (corr2 * penetration) * normal;
		v2 = v2 + (corr2 * vpen) * normal;			
}
