#ifndef _____dynamique_hpp______________________
#define _____dynamique_hpp______________________

#include <Inventor/nodes/SoGroup.h>

namespace dynamique {

/** Initialise la scne dynamique. Tous les objets prcdemments dclars sont dtruits, et tous les identifiants deviennent caducs. */
void init();

/** Insre un boule dans la scne dynamique, et renvoie un identifiant de cette boule. 
  \param parent le noeud parent dans le graphe inventor (typiquement, root)
  \param position position initiale de la boule
  \param vitesse vitessse initiale de la boule
  \param masse masse de la boule
  \param rayon rayon de la boule
  \param couleur couleur RVB de la boule
  Une masse nulle reprsente un objet inamovible. Une masse non nulle reprsente un objet mobile.
  */
int ajouteBoule( SoGroup* parent, SbVec3f position, SbVec3f vitesse, float masse, float rayon, SbVec3f couleur );

/** Insre un ressort entre deux boules.
  \param obj1 identifiant de la premire boule
  \param obj2 identifiant de la deuxime boule
  \param k raideur du ressort
  \param l0 longueur  vide du ressort
  \param nu facteur d'amortissement du ressort. C'est le rapport entre amortissement et raideur. La force d'amortissement vaut k*nu*(vitesse relative)
  */
int ajouteRessort( int obj1, int obj2, double k, double l0, double nu );

/** Mise  jour des positions et vitesses des objets dynamiques suivant un intervalle de temps dt */
void anime( double dt );

/** Rgle le vecteur de gravitation */
void setGravity( SbVec3f g );

/** Force la position d'une boule
  \param boule identifiant de la boule
  \param point nouvelle position de la boule
 */
void setPositionBoule( int boule, SbVec3f point );

}// dynamique
#endif
