
#include <math.h>
#include <GL/glut.h>
#include <stdlib.h>
#include <time.h>
#include <iostream>
#include "vec.h"
#include "dynamics.h"

using namespace std;



/////////////////////////////////////////
//  Camera methods
////////////////////////////////////////

float cameraAngle=M_PI*0.5f, distToOrigin = 10.0, cameraHeight = 3.0f;

void setCamera() {

		float x = distToOrigin*cos(cameraAngle);
		float y = cameraHeight;
		float z = distToOrigin*sin(cameraAngle);

		glLoadIdentity();
		gluLookAt(x, y, z, -x, y, -z,	0.0f,1.0f,0.0f);
}




/////////////////////////////////////////
//  Animation methods
////////////////////////////////////////

float oldTime=  0.0, baseTime = 0.005;
Dynamics dynamics;
int nbBalls = 2;

void updateAnimationParameters(){
		// compute elapsed time
		float newTime = (float)clock()/(float)CLOCKS_PER_SEC;
		float dt = (newTime-oldTime)/baseTime;
		oldTime = newTime;

		//dynamics.animate(dt);
}

void setDynamics(){
		// add a fixed ball
		Vec position0;
		Vec velocity0;
		float mass0 = 0.0;
		float radius0 = 0.1;
		int ball0 = dynamics.addBall( position0, velocity0, mass0, radius0); 

		// add another ball
		Vec position1 (0.0, 2.0, 0.0);
		Vec velocity1 (1.0, 0.0, 0.0);
		float mass1 = 1.0;
		float radius1 = 1.0;
		int ball1 = dynamics.addBall( position1, velocity1, mass1, radius1); 
		
}


////////////////////////////////////////
// Initialization methods
////////////////////////////////////////



void initScene() {
		// init GL states
		glClearColor(0.0f, 0.0f, 0.0f, 0.0f);				
		glClearDepth(1.0);													
		glDepthFunc(GL_LESS);												
		glEnable(GL_DEPTH_TEST);										
		glShadeModel(GL_SMOOTH);										
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

		// init dynamics
		setDynamics();
}




void changeSize(int w, int h)   {

		// Prevent a divide by zero, when window is too short
		// (you cant make a window of zero width).
		if(h == 0)
				h = 1;

		float ratio = 1.0f * w / h;
		// Reset the coordinate system before modifying
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();

		// Set the viewport to be the entire window
		glViewport(0, 0, w, h);

		// Set the clipping volume
		gluPerspective(45,ratio,1,1000);
		glMatrixMode(GL_MODELVIEW);
		setCamera();
}







///////////////////////////////////////////////
//  Draw methods
//////////////////////////////////////////////

void drawGround(){

		float backColor[] = {1.0, 0.0, 0.0};
		float frontColor[] = {0.0, 0.0, 1.0};
		
		glBegin(GL_QUADS);
		
		glColor3fv(frontColor);
		glVertex3f(-10.0f, 0.0f, -10.0f);
		glColor3fv(backColor);
		glVertex3f(-10.0f, 0.0f,  10.0f);
		glColor3fv(backColor);
		glVertex3f( 10.0f, 0.0f,  10.0f);
		glColor3fv(frontColor);
		glVertex3f( 10.0f, 0.0f, -10.0f);
		
		glEnd();
}



void drawSphere(){
		int nbStrips = 20;
		float stepVAngle = M_PI/nbStrips;
		
		int nbFacesPerStrip = 50;
		float stepHAngle = 2*M_PI/nbFacesPerStrip;

		for(int i=0 ; i<nbStrips ; i++) {
				float vAngle1 = i*stepVAngle - 0.5*M_PI;
				float vAngle2 = (i+1)*stepVAngle - 0.5*M_PI;
				
				// draw one strip
				float intensity1 = (float)i/nbStrips;
				float intensity2 = (float)(i+1)/nbStrips;
				glBegin(GL_QUAD_STRIP);
				for (int j=0 ; j<=nbFacesPerStrip ; j++){
						float hAngle = j*stepHAngle;						
						float pos1[] = {cos(hAngle)*cos(vAngle1), sin(vAngle1), sin(hAngle)*cos(vAngle1)};
						float pos2[] = {cos(hAngle)*cos(vAngle2), sin(vAngle2), sin(hAngle)*cos(vAngle2)};
						
						glNormal3fv(pos1);
						glColor3f(intensity1, 0.0, 1.0-intensity1);
						glVertex3fv(pos1);
						
						glNormal3fv(pos2);
						glColor3f(intensity2, 0.0, 1.0-intensity2);
						glVertex3fv(pos2);
				}
				glEnd();
		}
}


void drawBalls(){
		for (int i=0 ; i<nbBalls ; i++){
				glPushMatrix();
				Vec pos = dynamics.getBallPos(i);
				glTranslatef(pos.x, pos.y, pos.z);
				drawSphere();
				glPopMatrix();
		}
}

void renderScene(void) {
		// clear buffers
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

		// draw objects
		drawGround();
		drawBalls();

		// updateAnimationParameters
		updateAnimationParameters();
								
		// display rendering
		glutSwapBuffers();
}









///////////////////////////////////////
//  Interaction methods
///////////////////////////////////////


void specialKey(int key, int x_, int y_) {

		switch (key) {
		case GLUT_KEY_LEFT :
				cameraAngle += 0.1f;
				setCamera();  break;
		case GLUT_KEY_RIGHT :
				cameraAngle -=0.1f;
				setCamera();  break;
		case GLUT_KEY_UP :
				distToOrigin -= 0.2f;
				if (distToOrigin<1.5f) distToOrigin = 1.5f;
				setCamera();  break;
		case GLUT_KEY_DOWN :
				distToOrigin += 0.2f;
				setCamera();  break;
		case GLUT_KEY_PAGE_UP :
				cameraHeight += 0.1f;
				setCamera();  break;
		case GLUT_KEY_PAGE_DOWN :
				cameraHeight -= 0.1f;
				setCamera();  break;
		}
}




void commonKey(unsigned char key, int x_, int y_) {
		switch (key) {
		default:
				cout << key << endl; break;
		}
}






////////////////////////////////////
//  Main method
////////////////////////////////////

int main(int argc, char **argv) {

		// create window
		glutInit(&argc, argv);
		glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
		glutInitWindowPosition(100,100);
		glutInitWindowSize(800, 600);
		glutCreateWindow("Dynamics");

		// initialize OpenGL states
		initScene();

		// Bind Key events
		glutSpecialFunc(specialKey);
		glutKeyboardFunc(commonKey);

		// Bind rendering methods
		glutDisplayFunc(renderScene);
		glutIdleFunc(renderScene);

		glutReshapeFunc(changeSize);

		// run program
		glutMainLoop();

		return(0);
}
