#ifndef __VEC3__
#define __VEC3__


class Vec3 {
    public:
        Vec3(float x = 0, float y = 0, float z = 0);
        Vec3(const float *v);   // tableau de 3 valeurs

        
    // oprateurs de conversion :
    //---------------------------
        // oprateurs de conversion automatique :
        inline operator const float* () const { return v; }     // cast en const float*
        inline operator       float* ()       { return v; }     // cast en float*

    // oprations gomtriques :
    //--------------------------
        float norm() const;         // norme
        float norm2() const;        // norme au carr
        void  normalize();          // normalise le vecteur
        Vec3  normalized() const;   // retourne le vecteur normalis

        friend Vec3  vec(const Vec3 &a, const Vec3 &b);    // produit vectoriel
        friend float dot(const Vec3 &a, const Vec3 &b);    // produit scalaire
        friend Vec3  operator^(const Vec3 &a, const Vec3 &b) { return vec(a,b); }
        friend float operator|(const Vec3 &a, const Vec3 &b) { return dot(a,b); }
        
        friend float dist(Vec3 v1, Vec3 v2);    // distance euclidienne
        friend Vec3 triangleNormal(const Vec3 &a, const Vec3 &b, const Vec3 &c);    // normale du triangle (a,b,c), dans le sens trigo

    // oprateurs arithmtiques :
    //---------------------------
        // oprateurs arithmtiques unaires :
        friend Vec3 operator-(const Vec3 &a);

        // oprateurs arithmtiques binaires :
        friend Vec3 operator+(const Vec3 &a, const Vec3 &b);
        friend Vec3 operator-(const Vec3 &a, const Vec3 &b);
        friend Vec3 operator*(const Vec3 &a, const Vec3 &b);
        friend Vec3 operator/(const Vec3 &a, const Vec3 &b);
            
        // oprations avec un scalaire :
        friend Vec3 operator*(float s, const Vec3 &a);
        friend Vec3 operator*(const Vec3 &a, float s);
        friend Vec3 operator/(float s, const Vec3 &a);
        friend Vec3 operator/(const Vec3 &a, float s);
        
        // oprateurs de modification :
        Vec3& operator+=(const Vec3 &a);
        Vec3& operator-=(const Vec3 &a);
        Vec3& operator*=(const Vec3 &a);
        Vec3& operator/=(const Vec3 &a);
        Vec3& operator*=(float s);
        Vec3& operator/=(float s);
    
    // oprateurs de comparaison :
    //----------------------------
        friend bool operator==(const Vec3 &a, const Vec3 &b);   // comparaison  1e-10 prs
        friend bool operator!=(const Vec3 &a, const Vec3 &b);   // comparaison  1e-10 prs
        // les comparaisons suivantes sont le <et> boolen des comparaisons sur les composantes :
        friend bool operator>=(const Vec3 &a, const Vec3 &b);
        friend bool operator<=(const Vec3 &a, const Vec3 &b);
        friend bool operator>(const Vec3 &a, const Vec3 &b);
        friend bool operator<(const Vec3 &a, const Vec3 &b);
        friend bool operator>=(const Vec3 &a, float b);
        friend bool operator<=(const Vec3 &a, float b);
        friend bool operator>(const Vec3 &a, float b);
        friend bool operator<(const Vec3 &a, float b);
    
    // autres fonctions pratiques :
    //-----------------------------
        friend Vec3 abs(const Vec3 &a);
        friend Vec3 sign(const Vec3 &a);
        friend Vec3 floor(const Vec3 &a);
        friend Vec3 ceil(const Vec3 &a);
        friend Vec3 fract(const Vec3 &a);
        friend Vec3 min(const Vec3 &a, const Vec3 &b);
        friend Vec3 min(const Vec3 &a, float b);
        friend Vec3 max(const Vec3 &a, const Vec3 &b);
        friend Vec3 max(const Vec3 &a, float b);
        friend Vec3 clamp(const Vec3 &a, const Vec3 &min, const Vec3 &max);
        friend Vec3 clamp(const Vec3 &a, float min, float max);
        friend Vec3 mix(const Vec3 &a, const Vec3 &b, const Vec3 &alpha);     // (1-alpha) * a + alpha * b
        friend Vec3 mix(const Vec3 &a, const Vec3 &b, float alpha);           // (1-alpha) * a + alpha * b
        friend Vec3 step(const Vec3 &e, const Vec3 &a);
        friend Vec3 step(float e, const Vec3 &a);
        friend float min(const Vec3 &a);    // minimum des composantes
        friend float max(const Vec3 &a);    // maximum des composantes
    
    // oprations OpenGL :
    //--------------------
        void glVertex() const;    // effectue le glVertex correspondant
        void glNormal() const;    // effectue le glNormal correspondant
        
    public:
        union {
            struct { float x, y, z; };
            float v[3];
            };
    };

#endif

