
class Vec3{
public:
	float x;
	float y;
	float z;

public:
	Vec3() : x(0.0f), y(0.0f), z(0.0f) {}
	Vec3(float _x, float _y, float _z) : x(_x), y(_y), z(_z) {}
};


bool TriMesh::loadObj(const char * filename, bool normalize) {
	
  ifstream in(filename);
	char type;
	
  if (!in.is_open()) {
    return false;
	}

	vector<Vec3> vertices, normals;
	vector< pair<int, int> > faceIdx1, faceIdx2, faceIdx3;

	while (!in.eof()) {
		if (in.peek() == 'v') {
			in >> type;
			
			if (in.peek() == 'n'){
				in >> type;
			
				float x, y, z;
				in>> x; in>> y; in>> z;
				normals.push_back(Vec3(x,y,z));
				
			}	else if (in.peek() != 't') {
				float x, y, z;
				in>> x; in>> y; in>> z;
				vertices.push_back(Vec3(x,y,z));				
			}
			in.getline(s, 256);
			
		}	else if (in.peek() == 'f') {
			in >> type;
			
			int vertex1, vertex2, vertex3;
			int normal1, normal2, normal3;
			char tempByte;
			
			// vertex 1
			in>> vertex1; --vertex1;
			// separator
			in >> tempByte; in >> tempByte;				
			// normal 1
			in>> normal1; --normal1;
			
			// vertex 2
			in>> vertex2; --vertex2;
			// separator
			in >> tempByte; in >> tempByte;				
			// normal 2
			in>> normal2; --normal2;
			
			// vertex 3
			in>> vertex3; --vertex3;
			// separator
			in >> tempByte; in >> tempByte;				
			// normal 3
			in>> normal3; --normal3;
			
			// Store face
			pair<int, int> idx1 (vertex1, normal1);
			pair<int, int> idx2 (vertex2, normal2);
			pair<int, int> idx3 (vertex3, normal3);

			faceIdx1.push_back(idx1);
			faceIdx2.push_back(idx2);
			faceIdx3.push_back(idx3);
			
			in.getline(s, 256);
		} else {
			in.getline(s, 256);
		}
	}	
	in.close();


	// normalize object scale.
	if (normalize) {
		float minX=FLT_MAX;
		float maxX=-FLT_MAX;
		float minY=FLT_MAX;;
		float maxY=-FLT_MAX;
		float minZ=FLT_MAX;
		float maxZ=-FLT_MAX;

		// find bounding box coords
		int nbVertices = vertices.size();	
		for (int i=0 ; i<nbVertices ; i++){
			minX=(minX>vertices[i].x)?vertices[i].x:minX;				
			maxX=(maxX<vertices[i].x[0])?vertices[i].x:maxX;
			minY=(minY>vertices[i].y[1])?vertices[i].y:minY;				
			maxY=(maxY<vertices[i].y[1])?vertices[i].y:maxY;
			minZ=(minZ>vertices[i].z[2])?vertices[i].z:minZ;				
			maxZ=(maxZ<vertices[i].z[2])?vertices[i].z:maxZ;
		}

		// modify object vertices so that the maximum distance is unit size...
		Vec3 center (0.5*(maxX+minX), 0.5*(maxZ+minZ), 0.5*(maxZ+minZ));
		float norm = sqrt((maxX-minX)*(maxX-minX) + (maxY-minY)*(maxY-minY) + (maxZ-minZ)*(maxZ-minZ) );
	  float scaling=1.0/norm;
	
	  for (int i=0 ; i<nbVertices ; i++) {		  
	    vertices[i].x =(vertices[i].x - center.x)*scaling;
	    vertices[i].y =(vertices[i].y - center.y)*scaling;
	    vertices[i].z =(vertices[i].z - center.z)*scaling;
	  }
	} 

	return true;			
}


