
bool TriMesh::loadObj(const char * filename, bool normalize)
{
  ifstream in(filename);
	
  if (!in.is_open())
    return false;

  cout << filename<<endl;

  int nbChar=strlen(filename);
  string materialFileName(filename);
  nbChar-=3;
  int i=0;
  for (;i<nbChar;++i)
  {
	  materialFileName.at(i)=filename[i];
  }

  materialFileName.at(i++)='m';
  materialFileName.at(i++)='t';
  materialFileName.at(i)='l';

  ifstream materials(&(materialFileName[0]));
  bool hasMaterials=true;


  map<string,float> redD;
  map<string,float> greenD;
  map<string,float> blueD;
  map<string,float> alpha;

  char s[256];
  char type;
  float value;
  char currentMaterial[256];

	if (materials.is_open())
	{
		while (!materials.eof())
		{
			if (materials.peek() == 'n')
			{
				materials >> type;
				materials >> type;
				assert(type == 'e');
				materials >> type;
				assert(type == 'w');
				materials >> type;
				assert(type == 'm');
				materials >> type;
				assert(type == 't');
				materials >> type;
				assert(type == 'l');
				materials.getline(currentMaterial, 256);				}
			else
				if (materials.peek() == 'K')
				{
					materials >> type;
					materials >> type;
					if (type == 'd')
					{
						materials >> value;
						redD[currentMaterial] = value;
						materials >> value;
						greenD[currentMaterial] = value;
						materials >> value;
						blueD[currentMaterial] = value;
						materials.getline(s, 256);			
					}
					else
					{
						materials.getline(s, 256);			
					}
				}
				else if (materials.peek() == 'd')
				{
					materials >> type;
					materials >> value;
					alpha[currentMaterial] = value;
					materials.getline(s, 256);			
				}
				else
					materials.getline(s, 256);			
			}
	}
	else
		hasMaterials = false;

	materials.close();


	if (!hasMaterials)
	  cout <<"Materials not loaded"<<endl;
	else
	  cout <<"Materials loaded"<<endl;
	
	unsigned char currentMaterialRGBDiff[]={255, 255, 255};
	unsigned char currentMaterialAlpha = 255;
		


	while (!in.eof())
	{
		if (in.peek() == 'v')
		{
			in >> type;
			if (in.peek() != 't'&&in.peek() != 'n')
			{
				float x, y, z;
				in>> x;
				in>> y;
				in>> z;
				add_vertex(TriMesh::Point(x, y, z));
			}
			in.getline(s, 256);
		}
		else if (in.peek() == 'u')
		{
			if (hasMaterials)
			{
				in >> type;
				in >> type;
				assert(type == 's');
				in >> type;
				assert(type == 'e');
				in >> type;
				assert(type == 'm');
				in >> type;
				assert(type == 't');
				in >> type;
				assert(type == 'l');
				
				in.getline(currentMaterial, 256);
				
				currentMaterialRGBDiff[0] =(unsigned char)(255*redD[currentMaterial]);
				currentMaterialRGBDiff[1] =(unsigned char)(255*greenD[currentMaterial]);
				currentMaterialRGBDiff[2] =(unsigned char)(255*blueD[currentMaterial]);
				currentMaterialAlpha =(unsigned char)(255*alpha[currentMaterial]);
				
				printf("%s++\n %d %d %d %d\n",currentMaterial,currentMaterialRGBDiff[0],currentMaterialRGBDiff[1],currentMaterialRGBDiff[2],currentMaterialAlpha);
			}
			else
				in.getline(s, 256);
		}
		else if (in.peek() == 'f')
		{
			in >> type;
			
			if (in.peek() != 'n')
			{
				int vertex1, vertex2, vertex3;
				int tempData;
				char tempByte;
				
				in>> vertex1;
				--vertex1;
				if (in.peek() == '/')
				{
					// Normal
					in >> tempByte;
					in >> tempData;
					
					// material
					in >> tempByte;
					in >> tempData;
				}
				
				
				in>> vertex2;
				--vertex2;
				if (in.peek() == '/')
				{
					// Normal
					in >> tempByte;
					in >> tempData;
					
					// material
					in >> tempByte;
					in >> tempData;
				}
				
				
				in>> vertex3;
				--vertex3;
				if (in.peek() == '/')
				{
					// Normal
					in >> tempByte;
					in >> tempData;
					
					// material
					in >> tempByte;
					in >> tempData;
				}
				vector < TriMesh::VertexHandle>  face;
				face.push_back(TriMesh::VertexHandle(vertex1));
				face.push_back(TriMesh::VertexHandle(vertex2));
				face.push_back(TriMesh::VertexHandle(vertex3));
				FaceHandle fh = add_face(face);
				TriMesh::Color col;
				col[0] = currentMaterialRGBDiff[0];
				col[1] = currentMaterialRGBDiff[1];
				col[2] = currentMaterialRGBDiff[2];
				set_color(fh, col);
				property(_alphaProperty, fh) = currentMaterialAlpha;
				
				in.getline(s, 256);
				
				if (s[0] != 0)
				{
					if (s[1] != 0)
					{
						int i = 0;
						while (s[i] == 32)
							++i;
						
						int vertex4 = 0;
						if (isdigit(s[i]))
						{
							while (s[i] != '/')
							{
								vertex4 *= 10;
								vertex4 += s[i] - 48;// offset
								++i;
							}
						}
						--vertex4;
						vector < TriMesh::VertexHandle>  face2;
						face.push_back(TriMesh::VertexHandle(vertex3));
						face.push_back(TriMesh::VertexHandle(vertex4));
						face.push_back(TriMesh::VertexHandle(vertex1));
						add_face(face);
					}
				}
			}
		}
		else
		{
			in.getline(s, 256);
		}
	}
	
	in.close();

	float minX=FLT_MAX;
	float maxX=-FLT_MAX;
	float minY=FLT_MAX;;
	float maxY=-FLT_MAX;
	float minZ=FLT_MAX;
	float maxZ=-FLT_MAX;
		
		

	
	for (TriMesh::ConstVertexIter viter=vertices_begin();viter!=vertices_end();++viter)
	{
	  
	  const TriMesh::Point & p=point(viter.handle());
	  
	  minX=(minX>p[0])?p[0]:minX;				
	  maxX=(maxX<p[0])?p[0]:maxX;
	  minY=(minY>p[1])?p[1]:minY;				
	  maxY=(maxY<p[1])?p[1]:maxY;
	  minZ=(minZ>p[2])?p[2]:minZ;				
	  maxZ=(maxZ<p[2])?p[2]:maxZ;
	}
		
		
		
		
	TriMesh::Point maxP(maxX, maxY, maxZ);
	TriMesh::Point minP(minX, minY, minZ);
	
	_center=((maxP+minP)/2);

	
		

	if (normalize)
	{
	  float scaling=1.0/((maxP-minP).norm());
	
	  for (TriMesh::VertexIter viter2=vertices_begin();viter2!=vertices_end();++viter2)
	  {
		  
	    TriMesh::Point & p=point(viter2.handle());
	    p=(p-_center)*scaling;
	  }

	  _radius=1;
	  _center=TriMesh::Point(0,0,0);
	}
	else
	{
	  _radius=sqrt((maxX-minX)*(maxX-minX)+(maxY-minY)*(maxY-minY)+(maxZ-minZ)*(maxZ-minZ));
	}

	update_values();
	return true;			
}


