#include "dynamique.hpp"
#include "common.hpp"

#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoSphere.h>
#include <Inventor/nodes/SoMaterial.h>
#include "ressort.hpp"
#include "collision.hpp"

namespace dynamique{

typedef std::vector<SoTransform*> Transforms; ///< tableau de SoTransform* concerns par les variables d'tat

typedef std::vector<double> Masses; ///< tableau des masses associes aux variables d'tat

typedef std::vector<double> Rayons; ///< tableau des rayons des sphres

typedef std::vector<Ressort> Ressorts; ///< tableau de ressorts


Vector pos; ///< les positions
Vector vit; ///< les vitesses
Masses masses; ///< les masses
Masses inv_masses; ///< les inverses des masses
Transforms transforms;  ///< les noeuds concerns
Rayons rayons; ///< les rayons des boules
Ressorts ressorts;   ///< les ressorts
SbVec3f g(0,-10,0); ///< pesanteur;

SbVec3f sol_point(0,0,0);
SbVec3f sol_vitesse(0,0,0);
SbVec3f sol_normale(0,1,0);



void init()
{
	pos.clear();
	vit.clear();
	masses.clear();
	inv_masses.clear();
	transforms.clear();
	ressorts.clear();
	
}


int ajouteBoule( SoGroup* parent, SbVec3f x, SbVec3f v, float m, float r, SbVec3f c )
{
	// ajout au graphe inventor

	SoSeparator *boule = new SoSeparator;
	parent->addChild( boule );
	
	SoTransform* boule_pos = new SoTransform;
	boule->addChild( boule_pos );
	boule_pos->translation = x;
	
	SoMaterial* boule_mat = new SoMaterial;
	boule->addChild( boule_mat );
	boule_mat->diffuseColor = c;
	
	SoSphere* boule_forme = new SoSphere;
	boule->addChild( boule_forme );
	boule_forme->radius = r;
	
	// ajout aux variables dynamiques
	
	pos.push_back( x );
	vit.push_back( v );
	masses.push_back( m );
	if( m>0 ) inv_masses.push_back( 1/m ); 
	else inv_masses.push_back( 0 );
	rayons.push_back( r );
	transforms.push_back( boule_pos );
	
	return pos.size()-1;
}

int ajouteRessort( int obj1, int obj2, double k, double l0, double nu )
{
	ressorts.push_back( Ressort(obj1,obj2,k,l0,nu) );
	return ressorts.size();
}


void anime( double dt )
{
	int n = pos.size();
	Vector f( n, SbVec3f(0,0,0) );  // vecteur force initialement nul
	
	// -------- somme des forces ----------------------
	// poids
	for( int i=0; i<n; ++i )
	{
		f[i] += masses[i] * g;
	}
	// ressorts
	for( int i=0; i<ressorts.size(); ++i )
	{
		ressorts[i].ajouteForce(f,pos,vit);
	}
	
	//std::cout<<"forces: "<< f << endl;
	
	// accelerations (stockes dans f )
	for( int i=0; i<n; ++i )
	{
		f[i] *= inv_masses[i];
	}
	
	//std::cout<<"accelerations: "<< f << endl;
	
	// schma d'intgration
	for( int i=0; i<n; ++i )
	{
		pos[i] += dt * vit[i];
	}
	
	// traitement des collisions avec le sol
	//for( int i=0; i<n; ++i )
	//{
	//	collisionBoulePlan( pos[i], vit[i], rayons[i], inv_masses[i], sol_point, sol_vitesse, sol_normale, 0, 1);
	//}
	
	
	// appliquer les valeurs au graphe
	for( int i=0; i<n; ++i )
	{
		if(transforms[i]) transforms[i]->translation = pos[i];
	}
	
}

void setGravity( SbVec3f gravity ){ g = gravity; }

void setPositionBoule( int boule, SbVec3f point )
{
	assert( boule>=0 && boule<pos.size() );
	pos[boule] = point;
}


}
