/* Programme d'animation par modles physiques
Franois Faure, UJF-Grenoble, 2003
*/

/** \mainpage 
Ceci est la documentation HTML d'un programme d'animation par modles physiques. Elle ne documente pas le programme principal mais les diverses classes cres pour l'occasion. Elle est hlas incomplte car ne documente que les classes, et pas les mthodes externes.
*/

#include <stdlib.h>
#include <iostream.h>
#include <Inventor/SoDB.h>
#include <Inventor/Xt/SoXt.h>
#include <Inventor/Xt/viewers/SoXtExaminerViewer.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoDrawStyle.h>
#include <Inventor/nodes/SoEventCallback.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoTextureCoordinate2.h>
#include <Inventor/nodes/SoTexture2.h>
#include <Inventor/nodes/SoQuadMesh.h>
#include <Inventor/events/SoKeyboardEvent.h>
#include <Inventor/events/SoMouseButtonEvent.h>
#include <Inventor/events/SoLocation2Event.h>
#include <Inventor/sensors/SoTimerSensor.h>
#include "ivutils.h"                // lecture/ecriture de fichiers ...
#include "argumentParser.hpp"       // lecture de la ligne de commande
#include "dynamique/dynamique.hpp"  // librairie de dynamique

SoXtExaminerViewer *myViewer; // fenetre d'affichage
SoSeparator* root;           // racine du graphe de scne
SoDrawStyle* drawStyle;      // style d'affichage (plein, fil de fer...)
SoTimerSensor* animClock;    // horloge pour animation

float t=0;
float dt = 0.01;

SbVec3f gravity(0,-10,0);
int boule_fixe;
SbVec3f pointAttache(0,0,0);
SbVec3f pos_init(0,2,0);
SbVec3f vit_init(1,0,0);
double raideur_ressort = 1;
double longueur_ressort = 0;
double facteur_amortissement_ressort = 0.1;



// cration des objets dynamiques
void creeBoules()
{
	dynamique::init();
	dynamique::setGravity(gravity);
	
	boule_fixe = dynamique::ajouteBoule( root, pointAttache, SbVec3f(0,0,0), 0.0, 0.1, SbVec3f(0,1,0) ); // masse nulle => point fixe

	int boule = dynamique::ajouteBoule( root, pos_init, vit_init, 1.0, 1.0, SbVec3f(1,0,0) );

}


// mise  jour priodique dclenche par le SoTimerSensor* animClock
void
update(void *, SoSensor *)
{
	dynamique::anime(dt);
	t += dt;
}


// prise en compte des vnements clavier
void
myKeyPressCB(void *, SoEventCallback *eventCB)
{
	const SoEvent *event = eventCB->getEvent();

	if (SoKeyboardEvent::isKeyPressEvent (event,SoKeyboardEvent::W)) 
	// mode d'affichage
	{
		static int mode = 0;  // valeur initiale:0
   		if(mode==0){
			drawStyle->style = SoDrawStyleElement::LINES;
			mode=1;}
		else if(mode==1){
			drawStyle->style = SoDrawStyleElement::POINTS;
			mode=2;}
		else {
			drawStyle->style = SoDrawStyleElement::FILLED;
			mode=0;}
		eventCB->setHandled();
	} 
	else if (SO_KEY_PRESS_EVENT(event, S)) 
	// sauvegarde de la scene
	{
		printf("Sauvegarde dans le fichier scene.iv\n");
		iv::writeFile(root,"scene.iv");
		eventCB->setHandled();
	}
	else if (SO_KEY_PRESS_EVENT(event, SPACE)) 
	// avancer d'un pas de temps
	{
		// shift pour reculer
		if( event->wasShiftDown() ) dt = dt<0 ? dt : -dt;
		else dt = dt>0 ? dt : -dt;

		update(0,0);
	}
	else if (SO_KEY_PRESS_EVENT(event, R)) 
	// reinitialiser
	{
		t = 0;
		eventCB->setHandled();
	}
}


// prise en compte des boutons de la souris
bool button1=false; // vrai ssi bouton enfonc, initialement faux
bool button2=false;
bool button3=false;

void
myMousePressCB(void *, SoEventCallback *eventCB)
{
	const SoEvent *event = eventCB->getEvent();
	static bool running = false;

	if ( SoMouseButtonEvent::isButtonPressEvent(event,SoMouseButtonEvent:: BUTTON1) ){
		button1 = true;
		running = !running;
		if( running ) animClock->schedule();
		else animClock->unschedule();
		eventCB->setHandled();
	} 
	else if ( SoMouseButtonEvent::isButtonReleaseEvent(event,SoMouseButtonEvent:: BUTTON1) ){
		button1 = false;
	} 

	else if ( SoMouseButtonEvent::isButtonPressEvent(event,SoMouseButtonEvent:: BUTTON2) ){
		button2 = true;
	} 
	else if ( SoMouseButtonEvent::isButtonReleaseEvent(event,SoMouseButtonEvent:: BUTTON2) ){
		button2 = false;
	} 

	else if ( SoMouseButtonEvent::isButtonPressEvent(event,SoMouseButtonEvent:: BUTTON3) ){
		button3 = true;
	} 
	else if ( SoMouseButtonEvent::isButtonReleaseEvent(event,SoMouseButtonEvent:: BUTTON3) ){
		button3 = false;
	} 
}


// prise en compte du mouvement de la souris
void
myMouseMoveCB(void *, SoEventCallback *eventCB)
{
	const SoEvent *event = eventCB->getEvent();

	if(button2){
		SbVec2f location = event->getNormalizedPosition( myViewer->getViewportRegion() );
		std::cout<<"souris en "<<location[0]<<", "<<location[1]<<endl;
	}
	
}





main(int argc, char **argv)
{
	// paramtres par dfaut de la scne

	// lire la ligne de commande
	animal::ArgumentParser("Ce programme implmente une simulation physique.\nLes commandes interactives sont les suivantes:\nbouton gauche: dmarre/arrter\ntouche W: mode d'affichage\nTouche S: sauvegarde de la scene dans le fichier scene.iv\nTouche Espace: avancer d'un pas de temps\nShift+Espace: reculer d'un pas de temps\nTouche R: revenir  la position de dpart")
	.option(&dt,'t',"dt","pas de temps")
	.option(&pointAttache,'p',"point_attache","point d'attache de la boule")
	.option(&pos_init,'x',"position_initiale","positions initiale de la boule")
	.option(&raideur_ressort,'k',"raideur","raideur du(des) ressort(s)")
	.option(&longueur_ressort,'l',"longueur","longueur au repos du(des) ressort(s)")
	.option(&facteur_amortissement_ressort,'a',"amortissement_ressort","facteur d'amortissement du(des) ressort(s)")
	(argc,argv);

	// Initialiser Inventor et son interface graphique
	Widget myWindow = SoXt::init(argv[0]);
	if (myWindow == NULL) exit(1);

	// Creer la scne ---------------------------------------------

	// racine de la scne
	root = new SoSeparator;
	root->ref();

	// Visualisation de repere	
	SoSeparator* repere = iv::readFile("frame.iv");
	root->addChild( repere );

	// Noeud qui dfinit le style d'affichage
	drawStyle = new SoDrawStyle;
	root->addChild( drawStyle );

	// cration des boules
	creeBoules();	

	// ---fin creation scene ------------------------------------------

	// Un noeud qui lit les evenements (cablage des callback clavier et souris)
	SoEventCallback *myEventCB = new SoEventCallback;
	myEventCB->addEventCallback(
        	SoKeyboardEvent::getClassTypeId(), 
        	myKeyPressCB, NULL);
	myEventCB->addEventCallback(
        	SoMouseButtonEvent::getClassTypeId(), 
        	myMousePressCB, NULL);
	myEventCB->addEventCallback(
        	SoLocation2Event::getClassTypeId(), 
        	myMouseMoveCB, NULL);
	root->addChild(myEventCB);

	// Une horloge pour faire avancer le temps 
	animClock = new SoTimerSensor( update, NULL ); // callback d'animation
	animClock->setInterval( dt );

	// Crer un viewer
	myViewer = new SoXtExaminerViewer(myWindow);
	myViewer->setSceneGraph(root);
	myViewer->setTitle("Dynamique");
	myViewer->show();
	myViewer->viewAll();

	// Run
	SoXt::show(myWindow);
	SoXt::mainLoop();
}


